@@ -39,15 +39,15 @@ usage (int status)
 Usage: %s [ignored command line arguments]\n\
   or:  %s OPTION\n\
 "),
-	  program_name, program_name);
+          program_name, program_name);
   printf ("%s\n\n",
-	  _(EXIT_STATUS == EXIT_SUCCESS
-	    ? N_("Exit with a status code indicating success.")
-	    : N_("Exit with a status code indicating failure.")));
+          _(EXIT_STATUS == EXIT_SUCCESS
+            ? N_("Exit with a status code indicating success.")
+            : N_("Exit with a status code indicating failure.")));
   fputs (HELP_OPTION_DESCRIPTION, stdout);
   fputs (VERSION_OPTION_DESCRIPTION, stdout);
   printf (USAGE_BUILTIN_WARNING, PROGRAM_NAME);
-  emit_bug_reporting_address ();
+  emit_ancillary_info ();
   exit (status);
 }
 
@@ -67,11 +67,11 @@ main (int argc, char **argv)
   if (argc == 2)
     {
       if (STREQ (argv[1], "--help"))
-	usage (EXIT_STATUS);
+        usage (EXIT_STATUS);
 
       if (STREQ (argv[1], "--version"))
-	version_etc (stdout, PROGRAM_NAME, PACKAGE_NAME, Version, AUTHORS,
-		     (char *) NULL);
+        version_etc (stdout, PROGRAM_NAME, PACKAGE_NAME, Version, AUTHORS,
+                     (char *) NULL);
     }
 
   exit (EXIT_STATUS);
